@@ -12,7 +12,10 @@ pub use execute::{
     execute_rebase_plan, move_branches, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
     MergeConflictInfo, MergeConflictRemediation,
 };
-pub use plan::{BuildRebasePlanError, BuildRebasePlanOptions, RebasePlan, RebasePlanBuilder};
+pub use plan::{
+    BuildRebasePlanError, BuildRebasePlanOptions, RebasePlan, RebasePlanBuilder,
+    RebasePlanPermissions,
+};
 use tracing::instrument;
 
 use crate::core::task::{Resource, ResourcePool};
@@ -45,7 +48,8 @@ impl Resource for RepoResource {
             .repo
             .lock()
             .map_err(|_| eyre::eyre!("Poisoned mutex for RepoResource"))?;
-        repo.try_clone()
+        let repo = repo.try_clone()?;
+        Ok(repo)
     }
 }
 
