@@ -1,16 +1,17 @@
 use std::collections::{HashMap, HashSet};
-use std::fmt::Write;
+use std::fmt::{Display, Write};
 use std::ops::Sub;
 use std::path::PathBuf;
 use std::sync::Arc;
 
 use chashmap::CHashMap;
 use eden_dag::DagAlgorithm;
+use eyre::Context;
 use itertools::Itertools;
 use rayon::{prelude::*, ThreadPool};
 use tracing::{instrument, warn};
 
-use crate::core::dag::{commit_set_to_vec_unsorted, CommitSet, Dag};
+use crate::core::dag::{commit_set_to_vec, union_all, CommitSet, Dag};
 use crate::core::effects::{Effects, OperationType};
 use crate::core::formatting::{printable_styled_string, Pluralize};
 use crate::core::rewrite::{RepoPool, RepoResource};
@@ -23,11 +24,11 @@ pub enum OidOrLabel {
     Label(String),
 }
 
-impl ToString for OidOrLabel {
-    fn to_string(&self) -> String {
+impl Display for OidOrLabel {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::Oid(oid) => oid.to_string(),
-            Self::Label(label) => label.clone(),
+            OidOrLabel::Oid(oid) => write!(f, "{oid}"),
+            OidOrLabel::Label(label) => write!(f, "{label}"),
         }
     }
 }
@@ -50,7 +51,11 @@ pub enum RebaseCommand {
     },
 
     Merge {
-        /// The original merge commit to copy the commit message from.
+        /// If specified, the new merge commit will use this commit's message
+        /// and tree, rather than attempting a new merge.
+        replacement_commit_oid: Option<NonZeroOid>,
+
+        /// The original merge commit to copy the commit contents from.
         commit_oid: NonZeroOid,
 
         /// The other commits to merge into this one. This may be a list of
@@ -85,12 +90,13 @@ impl ToString for RebaseCommand {
     fn to_string(&self) -> String {
         match self {
             RebaseCommand::CreateLabel { label_name } => format!("label {}", label_name),
-            RebaseCommand::Reset { target } => format!("reset {}", target.to_string()),
+            RebaseCommand::Reset { target } => format!("reset {}", target),
             RebaseCommand::Pick {
                 original_commit_oid: _,
                 commit_to_apply_oid: commit_oid,
             } => format!("pick {}", commit_oid),
             RebaseCommand::Merge {
+                replacement_commit_oid: None,
                 commit_oid,
                 commits_to_merge,
             } => format!(
@@ -101,6 +107,21 @@ impl ToString for RebaseCommand {
                     .map(|commit_id| commit_id.to_string())
                     .join(" ")
             ),
+            RebaseCommand::Merge {
+                replacement_commit_oid: Some(replacement_commit_oid),
+                commit_oid,
+                commits_to_merge,
+            } => {
+                let other_parents = commits_to_merge
+                    .iter()
+                    .map(|parent| format!("-p {parent}"))
+                    .join(" ");
+                // FIXME: Untested, this probably doesn't work. Currently,
+                // on-disk rebases for merges with replacement commits are not
+                // generated by any command, so this code path is never
+                // triggered.
+                format!("exec git show -s --format=%B {replacement_commit_oid} | git commit-tree -p {commit_oid} {other_parents} {replacement_commit_oid}^{{tree}}")
+            }
             RebaseCommand::RegisterExtraPostRewriteHook => {
                 "exec git branchless hook-register-extra-post-rewrite-hook".to_string()
             }
@@ -120,19 +141,71 @@ impl ToString for RebaseCommand {
     }
 }
 
+/// A token representing that the rebase plan has been checked for validity.
+#[derive(Clone, Debug)]
+pub struct RebasePlanPermissions<'a> {
+    build_options: &'a BuildRebasePlanOptions,
+    allowed_commits: CommitSet,
+}
+
+impl<'a> RebasePlanPermissions<'a> {
+    /// Construct a new `RebasePlanPermissions`.
+    pub fn verify_rewrite_set(
+        dag: &Dag,
+        build_options: &'a BuildRebasePlanOptions,
+        commits: &CommitSet,
+    ) -> eyre::Result<Result<Self, BuildRebasePlanError>> {
+        // This isn't necessary for correctness, but helps to produce a better
+        // error message which indicates the magnitude of the issue.
+        let commits = dag.query().descendants(commits.clone())?;
+
+        let public_commits = dag.query_public_commits()?;
+        if !build_options.force_rewrite_public_commits {
+            let public_commits_to_move = public_commits.intersection(&commits);
+            if !public_commits_to_move.is_empty()? {
+                return Ok(Err(BuildRebasePlanError::MovePublicCommits {
+                    public_commits_to_move,
+                }));
+            }
+        }
+
+        let allowed_commits = dag.query().range(
+            commits.clone(),
+            commits.union(&dag.query_active_heads(&public_commits, &dag.observed_commits)?),
+        )?;
+        Ok(Ok(RebasePlanPermissions {
+            build_options,
+            allowed_commits,
+        }))
+    }
+
+    #[cfg(test)]
+    fn omnipotent_for_test(
+        dag: &Dag,
+        build_options: &'a BuildRebasePlanOptions,
+    ) -> eyre::Result<Self> {
+        Ok(Self {
+            build_options,
+            allowed_commits: dag.query().all()?,
+        })
+    }
+}
+
 /// Represents the commits (as OIDs) that will be used to build a rebase plan.
 #[derive(Clone, Debug)]
-struct ConstraintGraph<'repo> {
-    dag: &'repo Dag,
+struct ConstraintGraph<'a> {
+    dag: &'a Dag,
+    permissions: &'a RebasePlanPermissions<'a>,
 
     /// This is a mapping from `x` to `y` if `x` must be applied before `y`
     inner: HashMap<NonZeroOid, HashSet<NonZeroOid>>,
 }
 
-impl<'repo> ConstraintGraph<'repo> {
-    pub fn new(dag: &'repo Dag) -> Self {
+impl<'a> ConstraintGraph<'a> {
+    pub fn new(dag: &'a Dag, permissions: &'a RebasePlanPermissions) -> Self {
         Self {
             dag,
+            permissions,
             inner: HashMap::new(),
         }
     }
@@ -148,7 +221,7 @@ impl<'repo> ConstraintGraph<'repo> {
                 }
 
                 Constraint::MoveSubtree {
-                    parent_oid,
+                    parent_oids,
                     child_oid,
                 } => {
                     // remove previous (if any) constraints on commit
@@ -156,10 +229,12 @@ impl<'repo> ConstraintGraph<'repo> {
                         commits.remove(child_oid);
                     }
 
-                    self.inner
-                        .entry(*parent_oid)
-                        .or_default()
-                        .insert(*child_oid);
+                    for parent_oid in parent_oids {
+                        self.inner
+                            .entry(*parent_oid)
+                            .or_default()
+                            .insert(*child_oid);
+                    }
                 }
             }
         }
@@ -168,7 +243,7 @@ impl<'repo> ConstraintGraph<'repo> {
             .iter()
             .filter_map(|c| match c {
                 Constraint::MoveSubtree {
-                    parent_oid: _,
+                    parent_oids: _,
                     child_oid: _,
                 } => None,
                 Constraint::MoveChildren {
@@ -202,13 +277,13 @@ impl<'repo> ConstraintGraph<'repo> {
                         .difference(&self.dag.obsolete_commits)
                         .difference(&commits_to_move);
 
-                    for child_oid in commit_set_to_vec_unsorted(&source_children)? {
+                    for child_oid in commit_set_to_vec(&source_children)? {
                         self.inner.entry(parent_oid).or_default().insert(child_oid);
                     }
                 }
 
                 Constraint::MoveSubtree {
-                    parent_oid: _,
+                    parent_oids: _,
                     child_oid: _,
                 } => {
                     // do nothing; these were handled in the first pass
@@ -231,13 +306,13 @@ impl<'repo> ConstraintGraph<'repo> {
             .query()
             .children(CommitSet::from(current_oid))?
             .intersection(visible_commits);
-        let children_oids = commit_set_to_vec_unsorted(&children_oids)?;
+        let children_oids = commit_set_to_vec(&children_oids)?;
         for child_oid in children_oids {
             if self.commits_to_move().contains(&child_oid) {
                 continue;
             }
             acc.push(Constraint::MoveSubtree {
-                parent_oid: current_oid,
+                parent_oids: vec![current_oid],
                 child_oid,
             });
             self.collect_descendants(visible_commits, acc, child_oid)?;
@@ -296,6 +371,7 @@ impl<'repo> ConstraintGraph<'repo> {
                 self.check_for_cycles_helper(path, *child_oid)?;
             }
         }
+        path.pop();
         Ok(())
     }
 
@@ -310,22 +386,16 @@ impl<'repo> ConstraintGraph<'repo> {
         Ok(())
     }
 
-    fn check_for_rewriting_public_commits(
-        &self,
-        force: bool,
-    ) -> eyre::Result<Result<(), BuildRebasePlanError>> {
-        let commits_to_move = self
-            .inner
-            .values()
-            .flatten()
-            .copied()
-            .collect::<CommitSet>();
-        let public_commits = self.dag.query_public_commits()?;
-        let public_commits_to_move = commits_to_move.intersection(&public_commits);
-        let public_commits_to_move = self.dag.query().sort(&public_commits_to_move)?;
-        if !public_commits_to_move.is_empty()? && !force {
-            Ok(Err(BuildRebasePlanError::MovePublicCommits {
-                public_commits_to_move,
+    fn check_permissions(&self) -> eyre::Result<Result<(), BuildRebasePlanError>> {
+        let RebasePlanPermissions {
+            build_options: _,
+            allowed_commits,
+        } = &self.permissions;
+        let commits_to_move: CommitSet = self.commits_to_move().into_iter().collect();
+        let illegal_commits_to_move = commits_to_move.difference(allowed_commits);
+        if !illegal_commits_to_move.is_empty()? {
+            Ok(Err(BuildRebasePlanError::MoveIllegalCommits {
+                illegal_commits_to_move,
             }))
         } else {
             Ok(Ok(()))
@@ -347,7 +417,7 @@ impl<'repo> ConstraintGraph<'repo> {
                     children
                         .into_iter()
                         .map(move |child_oid| Constraint::MoveSubtree {
-                            parent_oid: unconstrained_oid,
+                            parent_oids: vec![unconstrained_oid],
                             child_oid,
                         })
                 })
@@ -420,8 +490,9 @@ struct BuildState<'repo> {
 /// Builder for a rebase plan. Unlike regular Git rebases, a `git-branchless`
 /// rebase plan can move multiple unrelated subtrees to unrelated destinations.
 #[derive(Clone, Debug)]
-pub struct RebasePlanBuilder<'repo> {
-    dag: &'repo Dag,
+pub struct RebasePlanBuilder<'a> {
+    dag: &'a Dag,
+    permissions: RebasePlanPermissions<'a>,
 
     /// The constraints specified by the caller.
     initial_constraints: Vec<Constraint>,
@@ -448,7 +519,7 @@ enum Constraint {
     /// Indicates that `child` and all of its descendants should be moved on top
     /// of `parent`.
     MoveSubtree {
-        parent_oid: NonZeroOid,
+        parent_oids: Vec<NonZeroOid>,
         child_oid: NonZeroOid,
     },
 }
@@ -486,6 +557,13 @@ pub enum BuildRebasePlanError {
         /// The public commits which the user was trying to move.
         public_commits_to_move: CommitSet,
     },
+
+    /// The user was trying to move commits that weren't verified before the
+    /// rebase plan was built. This probably indicates a bug in the code.
+    MoveIllegalCommits {
+        /// The illegal commits which the user was trying to move.
+        illegal_commits_to_move: CommitSet,
+    },
 }
 
 impl BuildRebasePlanError {
@@ -544,7 +622,7 @@ impl BuildRebasePlanError {
 You are trying to rewrite {}, such as: {}
 It is generally not advised to rewrite public commits, because your
 collaborators will have difficulty merging your changes.
-Retry with -f/--force to proceed anyways.",
+Retry with -f/--force-rewrite to proceed anyways.",
                     Pluralize {
                         determiner: None,
                         amount: public_commits_to_move.count()?,
@@ -556,16 +634,30 @@ Retry with -f/--force to proceed anyways.",
                     )?,
                 )?;
             }
+
+            BuildRebasePlanError::MoveIllegalCommits {
+                illegal_commits_to_move,
+            } => {
+                writeln!(
+                    effects.get_output_stream(),
+                    "\
+BUG: The following commits were planned to be moved but not verified:
+{:?}
+This is a bug. Please report it.",
+                    commit_set_to_vec(illegal_commits_to_move)
+                )?;
+            }
         }
         Ok(())
     }
 }
 
-impl<'repo> RebasePlanBuilder<'repo> {
+impl<'a> RebasePlanBuilder<'a> {
     /// Constructor.
-    pub fn new(dag: &'repo Dag) -> Self {
+    pub fn new(dag: &'a Dag, permissions: RebasePlanPermissions<'a>) -> Self {
         RebasePlanBuilder {
             dag,
+            permissions,
             initial_constraints: Default::default(),
             replacement_commits: Default::default(),
             touched_paths_cache: Default::default(),
@@ -657,11 +749,36 @@ impl<'repo> RebasePlanBuilder<'repo> {
 
                 match commits_to_merge {
                     Some(commits_to_merge) => {
-                        // All parents have been committed.
-                        acc.push(RebaseCommand::Merge {
-                            commit_oid: current_commit.get_oid(),
-                            commits_to_merge,
-                        });
+                        if acc.iter().any(|command| {
+                            matches!(command,
+                            RebaseCommand::Merge {
+                                replacement_commit_oid: _,
+                                commit_oid,
+                                commits_to_merge: _,
+                            } if *commit_oid == current_commit.get_oid())
+                        }) {
+                            // Above, we've checked to make sure that all parent
+                            // commits which are being moved have had their
+                            // labels applied. However, if the commit has
+                            // multiple parents which were *not* moved, then we
+                            // would visit this commit and attempt to reapply it
+                            // multiple times. The simplest fix is to not
+                            // reapply merge commits which have already been
+                            // applied.
+                            //
+                            // FIXME: O(n^2) algorithm.
+                            return Ok(acc);
+                        } else {
+                            // All parents have been committed.
+                            acc.push(RebaseCommand::Merge {
+                                replacement_commit_oid: self
+                                    .replacement_commits
+                                    .get(&current_commit.get_oid())
+                                    .copied(),
+                                commit_oid: current_commit.get_oid(),
+                                commits_to_merge,
+                            });
+                        }
                     }
 
                     None => {
@@ -763,14 +880,15 @@ impl<'repo> RebasePlanBuilder<'repo> {
     }
 
     /// Generate a sequence of rebase steps that cause the subtree at `source_oid`
-    /// to be rebased on top of `dest_oid`.
+    /// to be rebased on top of `dest_oids`.
     pub fn move_subtree(
         &mut self,
         source_oid: NonZeroOid,
-        dest_oid: NonZeroOid,
+        dest_oids: Vec<NonZeroOid>,
     ) -> eyre::Result<()> {
+        assert!(!dest_oids.is_empty());
         self.initial_constraints.push(Constraint::MoveSubtree {
-            parent_oid: dest_oid,
+            parent_oids: dest_oids,
             child_oid: source_oid,
         });
         Ok(())
@@ -798,7 +916,7 @@ impl<'repo> RebasePlanBuilder<'repo> {
         dest_oid: NonZeroOid,
     ) -> eyre::Result<()> {
         self.initial_constraints.push(Constraint::MoveSubtree {
-            parent_oid: dest_oid,
+            parent_oids: vec![dest_oid],
             child_oid: source_oid,
         });
         self.initial_constraints.push(Constraint::MoveChildren {
@@ -832,15 +950,8 @@ impl<'repo> RebasePlanBuilder<'repo> {
         effects: &Effects,
         pool: &ThreadPool,
         repo_pool: &ResourcePool<RepoResource>,
-        options: &BuildRebasePlanOptions,
     ) -> eyre::Result<Result<Option<RebasePlan>, BuildRebasePlanError>> {
-        let BuildRebasePlanOptions {
-            force_rewrite_public_commits,
-            dump_rebase_constraints,
-            dump_rebase_plan,
-            detect_duplicate_commits_via_patch_id,
-        } = options;
-        let mut constraints = ConstraintGraph::new(self.dag);
+        let mut constraints = ConstraintGraph::new(self.dag, &self.permissions);
         constraints.add_constraints(&self.initial_constraints)?;
         let mut state = BuildState {
             constraints,
@@ -850,6 +961,12 @@ impl<'repo> RebasePlanBuilder<'repo> {
 
         let (effects, _progress) = effects.start_operation(OperationType::BuildRebasePlan);
 
+        let BuildRebasePlanOptions {
+            force_rewrite_public_commits: _,
+            dump_rebase_constraints,
+            dump_rebase_plan,
+            detect_duplicate_commits_via_patch_id,
+        } = self.permissions.build_options;
         if *dump_rebase_constraints {
             // For test: don't print to `effects.get_output_stream()`, as it will
             // be suppressed.
@@ -871,10 +988,7 @@ impl<'repo> RebasePlanBuilder<'repo> {
         if let Err(err) = state.constraints.check_for_cycles(&effects) {
             return Ok(Err(err));
         }
-        if let Err(err) = state
-            .constraints
-            .check_for_rewriting_public_commits(*force_rewrite_public_commits)?
-        {
+        if let Err(err) = state.constraints.check_permissions()? {
             return Ok(Err(err));
         }
 
@@ -883,26 +997,33 @@ impl<'repo> RebasePlanBuilder<'repo> {
         let mut acc = Vec::new();
         let mut first_dest_oid = None;
         for constraint in roots {
-            let (parent_oid, child_oid) = match constraint {
+            let (parent_oids, child_oid) = match constraint {
                 Constraint::MoveSubtree {
-                    parent_oid,
+                    parent_oids,
                     child_oid,
-                } => (parent_oid, child_oid),
+                } => (parent_oids, child_oid),
                 Constraint::MoveChildren {
                     parent_of_oid: _,
                     children_of_oid: _,
                 } => eyre::bail!("BUG: Invalid constraint encountered while preparing rebase plan.\nThis should be unreachable."),
             };
-            first_dest_oid.get_or_insert(parent_oid);
+            let first_parent_oid = *parent_oids.first().unwrap();
+            first_dest_oid.get_or_insert(first_parent_oid);
             acc.push(RebaseCommand::Reset {
-                target: OidOrLabel::Oid(parent_oid),
+                target: OidOrLabel::Oid(first_parent_oid),
             });
 
             let upstream_patch_ids = if *detect_duplicate_commits_via_patch_id {
                 let (effects, _progress) =
                     effects.start_operation(OperationType::DetectDuplicateCommits);
                 self.get_upstream_patch_ids(
-                    &effects, pool, repo_pool, &repo, &mut state, child_oid, parent_oid,
+                    &effects,
+                    pool,
+                    repo_pool,
+                    &repo,
+                    &mut state,
+                    child_oid,
+                    &parent_oids,
                 )?
             } else {
                 Default::default()
@@ -911,7 +1032,7 @@ impl<'repo> RebasePlanBuilder<'repo> {
                 &effects,
                 &repo,
                 &mut state,
-                parent_oid,
+                first_parent_oid,
                 repo.find_commit_or_fail(child_oid)?,
                 &upstream_patch_ids,
                 acc,
@@ -950,6 +1071,7 @@ impl<'repo> RebasePlanBuilder<'repo> {
                     commit_to_apply_oid: commit_oid,
                 }
                 | RebaseCommand::Merge {
+                    replacement_commit_oid: _,
                     commit_oid,
                     commits_to_merge: _,
                 }
@@ -970,7 +1092,7 @@ impl<'repo> RebasePlanBuilder<'repo> {
         }
     }
 
-    /// Get the patch IDs for commits between `current_oid` and `dest_oid`,
+    /// Get the patch IDs for commits between `current_oid` and `dest_oids`,
     /// filtered to only the commits which might have the same patch ID as a
     /// commit being rebased.
     #[instrument]
@@ -982,22 +1104,28 @@ impl<'repo> RebasePlanBuilder<'repo> {
         repo: &Repo,
         state: &mut BuildState,
         current_oid: NonZeroOid,
-        dest_oid: NonZeroOid,
+        dest_oids: &[NonZeroOid],
     ) -> eyre::Result<HashSet<PatchId>> {
-        let merge_base_oid =
-            self.dag
-                .get_one_merge_base_oid(effects, repo, dest_oid, current_oid)?;
-        let merge_base_oid = match merge_base_oid {
-            None => return Ok(HashSet::new()),
-            Some(merge_base_oid) => merge_base_oid,
-        };
+        let merge_base_oids: Vec<CommitSet> = dest_oids
+            .iter()
+            .map(|dest_oid| {
+                let commit_set: CommitSet = [current_oid, *dest_oid].into_iter().collect();
+                self.dag.query().gca_all(commit_set)
+            })
+            .try_collect()?;
+        let merge_base_oids = union_all(&merge_base_oids);
 
         let touched_commit_oids: Vec<NonZeroOid> =
             state.constraints.commits_to_move().into_iter().collect();
 
-        let path = self
-            .dag
-            .get_range(effects, repo, merge_base_oid, dest_oid)?;
+        let path = {
+            let (effects, _progress) = effects.start_operation(OperationType::WalkCommits);
+            let _effects = effects;
+            self.dag
+                .query()
+                .range(merge_base_oids, dest_oids.iter().copied().collect())
+                .wrap_err("Calculating upstream commits")?
+        };
 
         let path = {
             self.filter_path_to_merge_base_commits(
@@ -1045,11 +1173,12 @@ impl<'repo> RebasePlanBuilder<'repo> {
         effects: &Effects,
         pool: &ThreadPool,
         repo_pool: &RepoPool,
-        repo: &'repo Repo,
-        path: Vec<NonZeroOid>,
+        repo: &'a Repo,
+        path: CommitSet,
         touched_commit_oids: Vec<NonZeroOid>,
-    ) -> eyre::Result<Vec<Commit<'repo>>> {
+    ) -> eyre::Result<Vec<Commit<'a>>> {
         let (effects, _progress) = effects.start_operation(OperationType::FilterCommits);
+        let path = commit_set_to_vec(&path)?;
 
         let touched_commits: Vec<Commit> = touched_commit_oids
             .into_iter()
@@ -1197,24 +1326,21 @@ mod tests {
             &references_snapshot,
         )?;
 
+        let build_options = BuildRebasePlanOptions {
+            force_rewrite_public_commits: true,
+            dump_rebase_constraints: false,
+            dump_rebase_plan: false,
+            detect_duplicate_commits_via_patch_id: true,
+        };
+        let permissions = RebasePlanPermissions::omnipotent_for_test(&dag, &build_options)?;
         let pool = ThreadPoolBuilder::new().build()?;
         let repo_pool = RepoPool::new(RepoResource {
             repo: Mutex::new(repo.try_clone()?),
         });
-        let mut builder = RebasePlanBuilder::new(&dag);
+        let mut builder = RebasePlanBuilder::new(&dag, permissions);
         let builder2 = builder.clone();
-        builder.move_subtree(test3_oid, test1_oid)?;
-        let result = builder.build(
-            &effects,
-            &pool,
-            &repo_pool,
-            &BuildRebasePlanOptions {
-                force_rewrite_public_commits: true,
-                dump_rebase_constraints: false,
-                dump_rebase_plan: false,
-                detect_duplicate_commits_via_patch_id: true,
-            },
-        )?;
+        builder.move_subtree(test3_oid, vec![test1_oid])?;
+        let result = builder.build(&effects, &pool, &repo_pool)?;
         let result = result.unwrap();
         let _ignored: Option<RebasePlan> = result;
         assert!(builder.touched_paths_cache.contains_key(&test1_oid));
@@ -1234,8 +1360,8 @@ mod tests {
         let test4_oid = git.commit_file("test4", 4)?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test4_oid, test1_oid)?;
-            builder.move_subtree(test4_oid, test2_oid)?;
+            builder.move_subtree(test4_oid, vec![test1_oid])?;
+            builder.move_subtree(test4_oid, vec![test2_oid])?;
             Ok(())
         })?;
 
@@ -1266,8 +1392,8 @@ mod tests {
         let test4_oid = git.commit_file("test4", 4)?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test3_oid, test1_oid)?;
-            builder.move_subtree(test4_oid, test1_oid)?;
+            builder.move_subtree(test3_oid, vec![test1_oid])?;
+            builder.move_subtree(test4_oid, vec![test1_oid])?;
             Ok(())
         })?;
 
@@ -1298,8 +1424,8 @@ mod tests {
         let test4_oid = git.commit_file("test4", 4)?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test4_oid, test1_oid)?;
-            builder.move_subtree(test3_oid, test1_oid)?;
+            builder.move_subtree(test4_oid, vec![test1_oid])?;
+            builder.move_subtree(test3_oid, vec![test1_oid])?;
             Ok(())
         })?;
 
@@ -1681,7 +1807,7 @@ mod tests {
         let _test5_oid = git.commit_file("test5", 5)?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test3_oid, test1_oid)?;
+            builder.move_subtree(test3_oid, vec![test1_oid])?;
             builder.move_commit(test4_oid, test1_oid)?;
             Ok(())
         })?;
@@ -1716,7 +1842,7 @@ mod tests {
         let _test5_oid = git.commit_file("test5", 5)?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test4_oid, test1_oid)?;
+            builder.move_subtree(test4_oid, vec![test1_oid])?;
             builder.move_commit(test3_oid, test1_oid)?;
             Ok(())
         })?;
@@ -1751,8 +1877,8 @@ mod tests {
         let test5_oid = git.commit_file("test5", 5)?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test3_oid, test5_oid)?;
-            builder.move_subtree(test5_oid, test2_oid)?;
+            builder.move_subtree(test3_oid, vec![test5_oid])?;
+            builder.move_subtree(test5_oid, vec![test2_oid])?;
             Ok(())
         })?;
 
@@ -1789,7 +1915,7 @@ mod tests {
         git.run(&["checkout", "HEAD~"])?;
 
         create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
-            builder.move_subtree(test3_oid, test2_oid)?;
+            builder.move_subtree(test3_oid, vec![test2_oid])?;
             Ok(())
         })?;
 
@@ -1817,7 +1943,7 @@ mod tests {
         x 9ea1b36 (rewritten as 2b47b505) create test5.txt
         |
         o 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
-        hint: there are 3 abandoned commits in your commit graph
+        hint: there is 1 abandoned commit in your commit graph
         hint: to fix this, run: git restack
         hint: disable this hint by running: git config --global branchless.hint.smartlogFixAbandoned false
         "###);
@@ -1850,21 +1976,19 @@ mod tests {
         let repo_pool = RepoPool::new(RepoResource {
             repo: Mutex::new(repo.try_clone()?),
         });
-        let mut builder = RebasePlanBuilder::new(&dag);
+
+        let build_options = BuildRebasePlanOptions {
+            force_rewrite_public_commits: false,
+            dump_rebase_constraints: false,
+            dump_rebase_plan: false,
+            detect_duplicate_commits_via_patch_id: true,
+        };
+        let permissions = RebasePlanPermissions::omnipotent_for_test(&dag, &build_options)?;
+        let mut builder = RebasePlanBuilder::new(&dag, permissions);
 
         builder_callback_fn(&mut builder)?;
 
-        let build_result = builder.build(
-            &effects,
-            &pool,
-            &repo_pool,
-            &BuildRebasePlanOptions {
-                force_rewrite_public_commits: false,
-                dump_rebase_constraints: false,
-                dump_rebase_plan: false,
-                detect_duplicate_commits_via_patch_id: true,
-            },
-        )?;
+        let build_result = builder.build(&effects, &pool, &repo_pool)?;
 
         let rebase_plan = match build_result {
             Ok(None) => return Ok(()),
