@@ -1,17 +1,60 @@
 use std::collections::{HashMap, HashSet};
 use std::convert::TryFrom;
-use std::ffi::OsString;
 use std::iter::FromIterator;
 use std::path::{Path, PathBuf};
 
-use eyre::Context;
+use bstr::ByteVec;
 use itertools::Itertools;
-use os_str_bytes::OsStringBytes;
+use thiserror::Error;
 use tracing::{instrument, warn};
 
 use super::oid::make_non_zero_oid;
 use super::status::FileMode;
-use super::{MaybeZeroOid, NonZeroOid, Repo};
+use super::{repo, MaybeZeroOid, NonZeroOid, Repo};
+
+#[derive(Debug, Error)]
+pub enum Error {
+    #[error("could not decode tree entry name: {0}")]
+    DecodeTreeEntryName(#[source] bstr::FromUtf8Error),
+
+    #[error(
+        "Tree entry was said to be an object of kind tree, but it could not be looked up: {oid}"
+    )]
+    NotATree { oid: NonZeroOid },
+
+    #[error("could not parse OID: {0}")]
+    ParseOid(#[source] eyre::Error),
+
+    #[error(transparent)]
+    FindTree(Box<repo::Error>),
+
+    #[error("could not find just-hydrated tree: {0}")]
+    FindHydratedTree(NonZeroOid),
+
+    #[error("could not read tree from path {path}: {source}")]
+    ReadTreeEntry { source: git2::Error, path: PathBuf },
+
+    #[error("could not construct tree builder: {0}")]
+    CreateTreeBuilder(#[source] git2::Error),
+
+    #[error("could not insert object {oid} with mode {file_mode:?} into tree builder: {source}")]
+    InsertTreeBuilderEntry {
+        source: git2::Error,
+        oid: NonZeroOid,
+        file_mode: FileMode,
+    },
+
+    #[error("could not read object at path {path} from tree builder: {source}")]
+    ReadTreeBuilderEntry { source: git2::Error, path: PathBuf },
+
+    #[error("could not delete object at path {path} from tree builder: {source}")]
+    DeleteTreeBuilderEntry { source: git2::Error, path: PathBuf },
+
+    #[error("could not build tree: {0}")]
+    BuildTree(#[source] git2::Error),
+}
+
+pub type Result<T> = std::result::Result<T, Error>;
 
 pub struct TreeEntry<'repo> {
     pub(super) inner: git2::TreeEntry<'repo>,
@@ -54,11 +97,14 @@ impl Tree<'_> {
     ///
     /// Note that the path isn't just restricted to entries of the current tree,
     /// i.e. you can use slashes in the provided path.
-    pub fn get_path(&self, path: &Path) -> eyre::Result<Option<TreeEntry>> {
+    pub fn get_path(&self, path: &Path) -> Result<Option<TreeEntry>> {
         match self.inner.get_path(path) {
             Ok(entry) => Ok(Some(TreeEntry { inner: entry })),
             Err(err) if err.code() == git2::ErrorCode::NotFound => Ok(None),
-            Err(err) => Err(err.into()),
+            Err(err) => Err(Error::ReadTreeEntry {
+                source: err,
+                path: path.to_owned(),
+            }),
         }
     }
 
@@ -66,7 +112,7 @@ impl Tree<'_> {
     ///
     /// Note that the path isn't just restricted to entries of the current tree,
     /// i.e. you can use slashes in the provided path.
-    pub fn get_oid_for_path(&self, path: &Path) -> eyre::Result<Option<MaybeZeroOid>> {
+    pub fn get_oid_for_path(&self, path: &Path) -> Result<Option<MaybeZeroOid>> {
         self.get_path(path)
             .map(|maybe_entry| maybe_entry.map(|entry| entry.inner.id().into()))
     }
@@ -80,7 +126,7 @@ fn get_changed_paths_between_trees_internal(
     current_path: &[PathBuf],
     lhs: Option<&git2::Tree>,
     rhs: Option<&git2::Tree>,
-) -> eyre::Result<()> {
+) -> Result<()> {
     let lhs_entries = lhs
         .map(|tree| tree.iter().collect_vec())
         .unwrap_or_default();
@@ -123,7 +169,7 @@ fn get_changed_paths_between_trees_internal(
             Tree(git2::Oid, i32),
         }
 
-        fn classify_entry(entry: Option<&git2::TreeEntry>) -> eyre::Result<ClassifiedEntry> {
+        fn classify_entry(entry: Option<&git2::TreeEntry>) -> Result<ClassifiedEntry> {
             let entry = match entry {
                 Some(entry) => entry,
                 None => return Ok(ClassifiedEntry::Absent),
@@ -136,24 +182,24 @@ fn get_changed_paths_between_trees_internal(
             }
         }
 
-        let get_tree = |oid| {
+        let get_tree = |oid: git2::Oid| -> Result<Tree> {
             let entry_oid = MaybeZeroOid::from(oid);
-            let entry_oid = NonZeroOid::try_from(entry_oid)?;
-            let entry_tree = repo.find_tree(entry_oid)?;
-            entry_tree.ok_or_else(|| {
-                eyre::eyre!(
-                    "Tree entry was said to be an object of kind tree, \
-                            but it could not be looked up: {:?}",
-                    oid
-                )
-            })
+            let entry_oid = NonZeroOid::try_from(entry_oid).map_err(Error::ParseOid)?;
+            let entry_tree = repo
+                .find_tree(entry_oid)
+                .map_err(Box::new)
+                .map_err(Error::FindTree)?;
+            entry_tree.ok_or(Error::NotATree { oid: entry_oid })
         };
 
-        let full_entry_path = || -> Vec<PathBuf> {
-            let entry_name: OsString = OsStringBytes::from_raw_vec(entry_name.to_vec()).unwrap();
+        let full_entry_path = || -> Result<Vec<PathBuf>> {
             let mut full_entry_path = current_path.to_vec();
-            full_entry_path.push(PathBuf::from(entry_name));
-            full_entry_path
+            let entry_name = entry_name
+                .to_vec()
+                .into_path_buf()
+                .map_err(Error::DecodeTreeEntryName)?;
+            full_entry_path.push(entry_name);
+            Ok(full_entry_path)
         };
         match (classify_entry(lhs_entry)?, classify_entry(rhs_entry)?) {
             (ClassifiedEntry::Absent, ClassifiedEntry::Absent) => {
@@ -168,21 +214,21 @@ fn get_changed_paths_between_trees_internal(
                     // Unchanged file, do nothing.
                 } else {
                     // Changed file.
-                    acc.push(full_entry_path());
+                    acc.push(full_entry_path()?);
                 }
             }
 
             (ClassifiedEntry::Absent, ClassifiedEntry::NotATree(_, _))
             | (ClassifiedEntry::NotATree(_, _), ClassifiedEntry::Absent) => {
                 // Added, removed, or changed file.
-                acc.push(full_entry_path());
+                acc.push(full_entry_path()?);
             }
 
             (ClassifiedEntry::Absent, ClassifiedEntry::Tree(tree_oid, _))
             | (ClassifiedEntry::Tree(tree_oid, _), ClassifiedEntry::Absent) => {
                 // A directory was added or removed. Add all entries from that
                 // directory.
-                let full_entry_path = full_entry_path();
+                let full_entry_path = full_entry_path()?;
                 let tree = get_tree(tree_oid)?;
                 get_changed_paths_between_trees_internal(
                     repo,
@@ -197,7 +243,7 @@ fn get_changed_paths_between_trees_internal(
             | (ClassifiedEntry::Tree(tree_oid, _), ClassifiedEntry::NotATree(_, _)) => {
                 // A file was changed into a directory. Add both the file and
                 // all subdirectory entries as changed entries.
-                let full_entry_path = full_entry_path();
+                let full_entry_path = full_entry_path()?;
                 let tree = get_tree(tree_oid)?;
                 get_changed_paths_between_trees_internal(
                     repo,
@@ -228,7 +274,7 @@ fn get_changed_paths_between_trees_internal(
 
                     (true, false) => {
                         // Only the directory changed, but none of its contents.
-                        acc.push(full_entry_path());
+                        acc.push(full_entry_path()?);
                     }
 
                     (false, true) => {
@@ -240,7 +286,7 @@ fn get_changed_paths_between_trees_internal(
                         get_changed_paths_between_trees_internal(
                             repo,
                             acc,
-                            &full_entry_path(),
+                            &full_entry_path()?,
                             Some(&lhs_tree.inner),
                             Some(&rhs_tree.inner),
                         )?;
@@ -249,7 +295,7 @@ fn get_changed_paths_between_trees_internal(
                     (false, false) => {
                         let lhs_tree = get_tree(lhs_tree_oid)?;
                         let rhs_tree = get_tree(rhs_tree_oid)?;
-                        let full_entry_path = full_entry_path();
+                        let full_entry_path = full_entry_path()?;
 
                         get_changed_paths_between_trees_internal(
                             repo,
@@ -273,7 +319,7 @@ pub fn get_changed_paths_between_trees(
     repo: &Repo,
     lhs: Option<&git2::Tree>,
     rhs: Option<&git2::Tree>,
-) -> eyre::Result<HashSet<PathBuf>> {
+) -> Result<HashSet<PathBuf>> {
     let mut acc = Vec::new();
     get_changed_paths_between_trees_internal(repo, &mut acc, &Vec::new(), lhs, rhs)?;
     let changed_paths: HashSet<PathBuf> = acc.into_iter().map(PathBuf::from_iter).collect();
@@ -301,7 +347,7 @@ pub fn hydrate_tree(
     repo: &Repo,
     tree: Option<&Tree>,
     entries: HashMap<PathBuf, Option<(NonZeroOid, FileMode)>>,
-) -> eyre::Result<NonZeroOid> {
+) -> Result<NonZeroOid> {
     let (file_entries, dir_entries) = {
         let mut file_entries: HashMap<PathBuf, Option<(NonZeroOid, FileMode)>> = HashMap::new();
         let mut dir_entries: HashMap<PathBuf, HashMap<PathBuf, Option<(NonZeroOid, FileMode)>>> =
@@ -328,71 +374,89 @@ pub fn hydrate_tree(
     let mut builder = repo
         .inner
         .treebuilder(tree)
-        .wrap_err("Instantiating tree builder")?;
+        .map_err(Error::CreateTreeBuilder)?;
     for (file_name, file_value) in file_entries {
         match file_value {
             Some((oid, file_mode)) => {
                 builder
                     .insert(&file_name, oid.inner, file_mode.into())
-                    .wrap_err_with(|| {
-                        format!(
-                            "Inserting file {:?} with OID: {:?}, file mode: {:?}",
-                            &file_name, oid, file_mode
-                        )
+                    .map_err(|err| Error::InsertTreeBuilderEntry {
+                        source: err,
+                        oid,
+                        file_mode,
                     })?;
             }
-            None => {
-                remove_entry_if_exists(&mut builder, &file_name)
-                    .wrap_err_with(|| format!("Removing deleted file: {:?}", &file_name))?;
-            }
+            None => remove_entry_if_exists(&mut builder, &file_name)?,
         }
     }
 
     for (dir_name, dir_value) in dir_entries {
-        let existing_dir_entry: Option<Tree> = match builder.get(&dir_name)? {
-            Some(existing_dir_entry)
-                if !existing_dir_entry.id().is_zero()
-                    && existing_dir_entry.kind() == Some(git2::ObjectType::Tree) =>
-            {
-                repo.find_tree(make_non_zero_oid(existing_dir_entry.id()))?
-            }
-            _ => None,
-        };
+        let existing_dir_entry: Option<Tree> =
+            match builder
+                .get(&dir_name)
+                .map_err(|err| Error::ReadTreeBuilderEntry {
+                    source: err,
+                    path: dir_name.to_owned(),
+                })? {
+                Some(existing_dir_entry)
+                    if !existing_dir_entry.id().is_zero()
+                        && existing_dir_entry.kind() == Some(git2::ObjectType::Tree) =>
+                {
+                    repo.find_tree(make_non_zero_oid(existing_dir_entry.id()))
+                        .map_err(Box::new)
+                        .map_err(Error::FindTree)?
+                }
+                _ => None,
+            };
         let new_entry_oid = hydrate_tree(repo, existing_dir_entry.as_ref(), dir_value)?;
 
         let new_entry_tree = repo
-            .find_tree(new_entry_oid)?
-            .ok_or_else(|| eyre::eyre!("Could not find just-hydrated tree: {:?}", new_entry_oid))?;
+            .find_tree(new_entry_oid)
+            .map_err(Box::new)
+            .map_err(Error::FindTree)?
+            .ok_or(Error::FindHydratedTree(new_entry_oid))?;
         if new_entry_tree.is_empty() {
-            remove_entry_if_exists(&mut builder, &dir_name)
-                .wrap_err_with(|| format!("Removing empty directory: {:?}", &dir_name))?;
+            remove_entry_if_exists(&mut builder, &dir_name)?;
         } else {
             builder
                 .insert(&dir_name, new_entry_oid.inner, git2::FileMode::Tree.into())
-                .wrap_err_with(|| {
-                    format!(
-                        "Inserting directory {:?} with OID: {:?}",
-                        &dir_name, new_entry_oid
-                    )
+                .map_err(|err| Error::InsertTreeBuilderEntry {
+                    source: err,
+                    oid: new_entry_oid,
+                    file_mode: FileMode::Tree,
                 })?;
         }
     }
 
-    let tree_oid = builder.write().wrap_err("Building tree")?;
+    let tree_oid = builder.write().map_err(Error::BuildTree)?;
     Ok(make_non_zero_oid(tree_oid))
 }
 
-pub fn make_empty_tree(repo: &Repo) -> eyre::Result<Tree> {
+pub fn make_empty_tree(repo: &Repo) -> Result<Tree> {
     let tree_oid = hydrate_tree(repo, None, Default::default())?;
     repo.find_tree_or_fail(tree_oid)
+        .map_err(Box::new)
+        .map_err(Error::FindTree)
 }
 
 /// `libgit2` raises an error if the entry isn't present, but that's often not
 /// an error condition here. We may be referring to a created or deleted path,
 /// which wouldn't exist in one of the pre-/post-patch trees.
-fn remove_entry_if_exists(builder: &mut git2::TreeBuilder, name: &Path) -> eyre::Result<()> {
-    if builder.get(&name)?.is_some() {
-        builder.remove(&name)?;
+fn remove_entry_if_exists(builder: &mut git2::TreeBuilder, name: &Path) -> Result<()> {
+    if builder
+        .get(&name)
+        .map_err(|err| Error::ReadTreeBuilderEntry {
+            source: err,
+            path: name.to_owned(),
+        })?
+        .is_some()
+    {
+        builder
+            .remove(&name)
+            .map_err(|err| Error::DeleteTreeBuilderEntry {
+                source: err,
+                path: name.to_owned(),
+            })?;
     }
     Ok(())
 }
@@ -401,10 +465,10 @@ fn remove_entry_if_exists(builder: &mut git2::TreeBuilder, name: &Path) -> eyre:
 ///
 /// If a provided path does not appear in the tree at all, then it's ignored.
 #[instrument]
-pub fn dehydrate_tree(repo: &Repo, tree: &Tree, paths: &[&Path]) -> eyre::Result<NonZeroOid> {
+pub fn dehydrate_tree(repo: &Repo, tree: &Tree, paths: &[&Path]) -> Result<NonZeroOid> {
     let entries: HashMap<PathBuf, Option<(NonZeroOid, FileMode)>> = paths
         .iter()
-        .map(|path| -> eyre::Result<(PathBuf, _)> {
+        .map(|path| -> Result<(PathBuf, _)> {
             let key = path.to_path_buf();
             match tree.inner.get_path(path) {
                 Ok(tree_entry) => {
@@ -415,7 +479,10 @@ pub fn dehydrate_tree(repo: &Repo, tree: &Tree, paths: &[&Path]) -> eyre::Result
                     Ok((key, value))
                 }
                 Err(err) if err.code() == git2::ErrorCode::NotFound => Ok((key, None)),
-                Err(err) => Err(err.into()),
+                Err(err) => Err(Error::ReadTreeEntry {
+                    source: err,
+                    path: key,
+                }),
             }
         })
         .try_collect()?;
