@@ -332,13 +332,14 @@ stderr:
     /// Clone this repository into the `target` repository (which must not have
     /// been initialized).
     pub fn clone_repo_into(&self, target: &Git, additional_args: &[&str]) -> eyre::Result<()> {
+        let remote = format!("file://{}", self.repo_path.to_str().unwrap());
         let args = {
             let mut args = vec![
                 "clone",
                 // For Windows in CI.
                 "-c",
                 "core.autocrlf=false",
-                self.repo_path.to_str().unwrap(),
+                &remote,
                 target.repo_path.to_str().unwrap(),
             ];
             args.extend(additional_args.iter());
@@ -423,14 +424,16 @@ stderr:
     /// Get a `Repo` object for this repository.
     #[instrument]
     pub fn get_repo(&self) -> eyre::Result<Repo> {
-        Repo::from_dir(&self.repo_path)
+        let repo = Repo::from_dir(&self.repo_path)?;
+        Ok(repo)
     }
 
     /// Get the version of the Git executable.
     #[instrument]
     pub fn get_version(&self) -> eyre::Result<GitVersion> {
         let (version_str, _stderr) = self.run(&["version"])?;
-        version_str.parse()
+        let version = version_str.parse()?;
+        Ok(version)
     }
 
     /// Get the `GitRunInfo` to use for this repository.
