@@ -70,13 +70,13 @@ use crate::commands::smartlog::smartlog;
 use crate::opts::{MoveOptions, Revset};
 use crate::revset::resolve_commits;
 use lib::core::config::get_restack_preserve_timestamps;
-use lib::core::dag::{commit_set_to_vec_unsorted, union_all, CommitSet, Dag};
+use lib::core::dag::{commit_set_to_vec, union_all, CommitSet, Dag};
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
 use lib::core::rewrite::{
     execute_rebase_plan, find_abandoned_children, find_rewrite_target, move_branches,
     BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
-    MergeConflictRemediation, RebasePlanBuilder, RepoPool, RepoResource,
+    MergeConflictRemediation, RebasePlanBuilder, RebasePlanPermissions, RepoPool, RepoResource,
 };
 use lib::git::{GitRunInfo, NonZeroOid, Repo};
 
@@ -102,7 +102,7 @@ fn restack_commits(
     };
     // Don't use `sort_commit_set` since the set of obsolete commits may be very
     // large and we'll be throwing away most of them.
-    let commits = commit_set_to_vec_unsorted(&commit_set)?;
+    let commits = commit_set_to_vec(&commit_set)?;
 
     let public_commits = dag.query_public_commits()?;
     let active_heads = dag.query_active_heads(
@@ -136,17 +136,36 @@ fn restack_commits(
     };
 
     let rebase_plan = {
-        let mut builder = RebasePlanBuilder::new(dag);
+        let permissions = match RebasePlanPermissions::verify_rewrite_set(
+            dag,
+            build_options,
+            &rebases
+                .iter()
+                .flat_map(
+                    |RebaseInfo {
+                         dest_oid: _,
+                         abandoned_child_oids,
+                     }| abandoned_child_oids.iter().copied(),
+                )
+                .collect(),
+        )? {
+            Ok(permissions) => permissions,
+            Err(err) => {
+                err.describe(effects, &repo)?;
+                return Ok(ExitCode(1));
+            }
+        };
+        let mut builder = RebasePlanBuilder::new(dag, permissions);
         for RebaseInfo {
             dest_oid,
             abandoned_child_oids,
         } in rebases
         {
             for child_oid in abandoned_child_oids {
-                builder.move_subtree(child_oid, dest_oid)?;
+                builder.move_subtree(child_oid, vec![dest_oid])?;
             }
         }
-        match builder.build(effects, thread_pool, repo_pool, build_options)? {
+        match builder.build(effects, thread_pool, repo_pool)? {
             Ok(Some(rebase_plan)) => rebase_plan,
             Ok(None) => {
                 writeln!(
@@ -289,7 +308,7 @@ pub fn restack(
         None
     } else {
         Some(
-            commit_set_to_vec_unsorted(&union_all(&commit_sets))?
+            commit_set_to_vec(&union_all(&commit_sets))?
                 .into_iter()
                 .collect(),
         )
