@@ -11,6 +11,7 @@ use std::fs::File;
 use std::io::Write as OtherWrite;
 use std::time::SystemTime;
 
+use bstr::{ByteSlice, ByteVec};
 use chrono::Local;
 use dialoguer::Editor;
 use eden_dag::DagAlgorithm;
@@ -27,7 +28,7 @@ use lib::core::formatting::{printable_styled_string, Glyphs, Pluralize};
 use lib::core::node_descriptors::{render_node_descriptors, CommitOidDescriptor, NodeObject};
 use lib::core::rewrite::{
     execute_rebase_plan, BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
-    RebasePlanBuilder, RepoResource,
+    RebasePlanBuilder, RebasePlanPermissions, RepoResource,
 };
 use lib::git::{message_prettify, Commit, GitRunInfo, MaybeZeroOid, NonZeroOid, Repo};
 
@@ -40,6 +41,9 @@ pub enum InitialCommitMessages {
     /// The user wants to start with an empty (or template) message.
     Discard,
 
+    /// The user wants to fixup a commit.
+    FixUp(Revset),
+
     /// The user provided explicit messages.
     Messages(Vec<String>),
 }
@@ -71,6 +75,61 @@ pub fn reword(
         Some(commits) => commits,
         None => return Ok(ExitCode(1)),
     };
+    let build_options = BuildRebasePlanOptions {
+        force_rewrite_public_commits,
+        dump_rebase_constraints: false,
+        dump_rebase_plan: false,
+        detect_duplicate_commits_via_patch_id: false,
+    };
+    let permissions = match RebasePlanPermissions::verify_rewrite_set(
+        &dag,
+        &build_options,
+        &commits.iter().map(|commit| commit.get_oid()).collect(),
+    )? {
+        Ok(permissions) => permissions,
+        Err(err) => {
+            err.describe(effects, &repo)?;
+            return Ok(ExitCode(1));
+        }
+    };
+
+    let messages = match messages {
+        InitialCommitMessages::Discard | InitialCommitMessages::Messages(_) => messages,
+        InitialCommitMessages::FixUp(revset) => {
+            let commits_to_fixup =
+                resolve_commits_from_hashes(&repo, &mut dag, effects, vec![revset.clone()])?
+                    .unwrap_or_default();
+            let commit_to_fixup = match commits_to_fixup.as_slice() {
+                [commit_to_fixup] => {
+                    let commits: CommitSet = commits.iter().map(|c| c.get_oid()).collect();
+                    if !dag
+                        .query()
+                        .common_ancestors(commits)?
+                        .contains(&commit_to_fixup.get_oid().into())?
+                    {
+                        writeln!(
+                            effects.get_error_stream(),
+                            "The commit supplied to --fixup must be an ancestor of all commits being reworded.\nAborting.",
+                        )?;
+                        return Ok(ExitCode(1));
+                    }
+                    commit_to_fixup
+                }
+                commits => {
+                    writeln!(
+                        effects.get_error_stream(),
+                        "--fixup expects exactly 1 commit, but '{}' evaluated to {}.\nAborting.",
+                        revset,
+                        commits.len()
+                    )?;
+                    return Ok(ExitCode(1));
+                }
+            };
+            let message = commit_to_fixup.get_summary()?.to_vec();
+            let message = format!("fixup! {}", message.into_string_lossy());
+            InitialCommitMessages::Messages(vec![message])
+        }
+    };
 
     #[instrument]
     fn edit_message_fn_inner(
@@ -168,51 +227,21 @@ pub fn reword(
         }
     };
 
-    let subtree_roots = find_subtree_roots(&repo, &dag, &commits)?;
-
     let rebase_plan = {
         let pool = ThreadPoolBuilder::new().build()?;
         let repo_pool = RepoResource::new_pool(&repo)?;
-        let mut builder = RebasePlanBuilder::new(&dag);
-
-        for root_commit in subtree_roots {
-            let only_parent_id = root_commit.get_only_parent().map(|parent| parent.get_oid());
-            let only_parent_id = match only_parent_id {
-                Some(only_parent_id) => only_parent_id,
-                None => {
-                    writeln!(
-                        effects.get_error_stream(),
-                        "Refusing to reword commit {}, which has {} parents.\n\
-                        Rewording is only supported for commits with 1 parent.\n\
-                        Aborting.",
-                        root_commit.get_oid(),
-                        root_commit.get_parents().len(),
-                    )?;
-                    return Ok(ExitCode(1));
-                }
-            };
-            builder.move_subtree(root_commit.get_oid(), only_parent_id)?;
-        }
+        let mut builder = RebasePlanBuilder::new(&dag, permissions);
 
         for commit in commits.iter() {
             let message = messages.get(&commit.get_oid()).unwrap();
             // This looks funny, but just means "leave everything but the message as is"
             let replacement_oid =
                 commit.amend_commit(None, None, None, Some(message.as_str()), None)?;
+            builder.move_subtree(commit.get_oid(), commit.get_parent_oids())?;
             builder.replace_commit(commit.get_oid(), replacement_oid)?;
         }
 
-        match builder.build(
-            effects,
-            &pool,
-            &repo_pool,
-            &BuildRebasePlanOptions {
-                force_rewrite_public_commits,
-                dump_rebase_constraints: false,
-                dump_rebase_plan: false,
-                detect_duplicate_commits_via_patch_id: false,
-            },
-        )? {
+        match builder.build(effects, &pool, &repo_pool)? {
             Ok(Some(rebase_plan)) => rebase_plan,
             Ok(None) => {
                 eyre::bail!(
@@ -258,13 +287,7 @@ pub fn reword(
         }
         ExecuteRebasePlanResult::Succeeded {
             rewritten_oids: None,
-        } => {
-            writeln!(
-                effects.get_error_stream(),
-                "BUG: Succeeded rewording commits via on-disk rebase? But reword should be rebasing in-memory!"
-            )?;
-            ExitCode(1)
-        }
+        } => ExitCode(0),
         ExecuteRebasePlanResult::DeclinedToMerge { merge_conflict: _ } => {
             writeln!(
                 effects.get_error_stream(),
@@ -343,6 +366,9 @@ fn prepare_messages(
         InitialCommitMessages::Discard => {
             (get_commit_template(repo)?.unwrap_or_default(), true, true)
         }
+        InitialCommitMessages::FixUp(_) => {
+            eyre::bail!("BUG: Fixup should have already been handled!")
+        }
         InitialCommitMessages::Messages(ref messages) => {
             let message = messages.clone().join("\n\n");
             let message = message.trim();
@@ -384,7 +410,7 @@ fn prepare_messages(
         let original_message = commit
             .get_message_raw()?
             .to_str()
-            .ok_or_else(|| {
+            .with_context(|| {
                 eyre::eyre!(
                     "Could not decode commit message for commit: {:?}",
                     commit.get_oid()
