@@ -9,6 +9,7 @@ use std::mem::swap;
 use std::time::SystemTime;
 
 use console::style;
+use eden_dag::DagAlgorithm;
 use lib::core::config::{get_hint_enabled, print_hint_suppression_notice, Hint};
 use lib::core::repo_ext::RepoExt;
 use lib::core::rewrite::find_rewrite_target;
@@ -39,7 +40,7 @@ mod graph {
     use lib::core::gc::mark_commit_reachable;
     use tracing::instrument;
 
-    use lib::core::dag::{commit_set_to_vec_unsorted, CommitSet, Dag};
+    use lib::core::dag::{commit_set_to_vec, CommitSet, Dag};
     use lib::core::effects::{Effects, OperationType};
     use lib::core::eventlog::{EventCursor, EventReplayer};
     use lib::core::node_descriptors::NodeObject;
@@ -175,7 +176,7 @@ mod graph {
             let mut links = Vec::new();
             for child_oid in non_main_node_oids {
                 let parent_vertexes = dag.query().parents(CommitSet::from(*child_oid))?;
-                let parent_oids = commit_set_to_vec_unsorted(&parent_vertexes)?;
+                let parent_oids = commit_set_to_vec(&parent_vertexes)?;
                 for parent_oid in parent_oids {
                     if graph.contains_key(&parent_oid) {
                         links.push((*child_oid, parent_oid))
@@ -240,7 +241,7 @@ mod graph {
             };
 
             let active_heads = dag.query_active_heads(&public_commits, &observed_commits)?;
-            for oid in commit_set_to_vec_unsorted(&active_heads)? {
+            for oid in commit_set_to_vec(&active_heads)? {
                 mark_commit_reachable(repo, oid)?;
             }
 
@@ -654,19 +655,21 @@ pub fn smartlog(
     }
 
     if !show_hidden_commits && get_hint_enabled(&repo, Hint::SmartlogFixAbandoned)? {
-        let num_abandoned_children = graph
+        let commits_with_abandoned_children: CommitSet = graph
             .nodes
             .iter()
             .filter_map(|(oid, node)| {
                 if node.is_obsolete
                     && find_rewrite_target(&event_replayer, event_cursor, *oid).is_some()
                 {
-                    Some(node.children.iter())
+                    Some(*oid)
                 } else {
                     None
                 }
             })
-            .count();
+            .collect();
+        let children = dag.query().children(commits_with_abandoned_children)?;
+        let num_abandoned_children = children.difference(&dag.obsolete_commits).count()?;
         if num_abandoned_children > 0 {
             writeln!(
                 effects.get_output_stream(),
