@@ -5,17 +5,19 @@ use std::sync::{Arc, Mutex};
 
 use crate::util::trim_lines;
 
-use git_branchless::commands::undo::testing::{select_past_event, undo_events};
+use git_branchless::commands::testing::undo::{select_past_event, undo_events};
 use git_branchless::tui::testing::{screen_to_string, CursiveTestingBackend, CursiveTestingEvent};
 use lib::core::dag::Dag;
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
 use lib::core::formatting::Glyphs;
+use lib::core::repo_ext::RepoExt;
 use lib::git::{GitRunInfo, GitVersion, Repo};
 use lib::testing::{make_git, Git};
 
 use cursive::event::Key;
 use cursive::CursiveRunnable;
+use lib::util::ExitCode;
 
 fn run_select_past_event(
     repo: &Repo,
@@ -59,7 +61,7 @@ fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<(isize,
         env: git.get_base_env(0).into_iter().collect(),
     };
 
-    let exit_code = undo_events(
+    let ExitCode(exit_code) = undo_events(
         &mut in_,
         &Effects::new_from_buffer_for_test(glyphs, &stdout, &stderr),
         &repo,
@@ -404,18 +406,13 @@ fn test_undo_move_refs() -> eyre::Result<()> {
         let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
-        1. Check out from 96d1c37 create test2.txt
-                       to 62fc20d create test1.txt
-        2. Hide commit 96d1c37 create test2.txt
+        1. Hide commit 96d1c37 create test2.txt
 
-        3. Move branch master from 96d1c37 create test2.txt
+        2. Move branch master from 96d1c37 create test2.txt
                                 to 62fc20d create test1.txt
+        3. Check out from 96d1c37 create test2.txt
+                       to 62fc20d create test1.txt
         Confirm? [yN] branchless: running command: <git-executable> checkout 62fc20d2a290daea0d52bdc2ed2ad4be6491010e --detach
-        branchless: running command: <git-executable> branchless smartlog
-        :
-        @ 62fc20d create test1.txt
-        |
-        O 96d1c37 (master) create test2.txt
         Applied 3 inverse events.
         "###);
         assert_eq!(exit_code, 0);
@@ -630,21 +627,17 @@ fn test_undo_doesnt_make_working_dir_dirty() -> eyre::Result<()> {
         let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
-        1. Check out from 62fc20d create test1.txt
-                       to f777ecc create initial.txt
-        2. Delete branch bar at 62fc20d create test1.txt
+        1. Delete branch bar at 62fc20d create test1.txt
 
-        3. Hide commit 62fc20d create test1.txt
+        2. Hide commit 62fc20d create test1.txt
 
-        4. Move branch master from 62fc20d create test1.txt
+        3. Move branch master from 62fc20d create test1.txt
                                 to f777ecc create initial.txt
+        4. Check out from 62fc20d create test1.txt
+                       to f777ecc create initial.txt
         5. Delete branch foo at f777ecc create initial.txt
 
         Confirm? [yN] branchless: running command: <git-executable> checkout f777ecc9b0db5ed372b2615695191a8a17f79f24 --detach
-        branchless: running command: <git-executable> branchless smartlog
-        @ f777ecc (foo) create initial.txt
-        |
-        O 62fc20d (bar, master) create test1.txt
         Applied 5 inverse events.
         "###);
         assert_eq!(exit_code, 0);
@@ -706,7 +699,7 @@ fn test_git_bisect_produces_empty_event() -> eyre::Result<()> {
     ┌──────────────────────────────────────────────────────┤ Events ├──────────────────────────────────────────────────────┐
     │Repo after transaction 3 (event 4). Press 'h' for help, 'q' to quit.                                                  │
     │1. Empty event for BISECT_HEAD                                                                                        │
-    │   This may be an unsupported use-case; see https://git.io/J0b7z                                                      │
+    │   This may be an unsupported use-case; see https://github.com/arxanas/git-branchless/issues/57                       │
     └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
     "###);
 
@@ -796,15 +789,14 @@ fn test_undo_garbage_collected_commit() -> eyre::Result<()> {
         let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
-        1. Check out from 62fc20d create test1.txt
-                       to <commit not available: 96d1c37a3d4363611c49f7e52186e189a04c531f>
+        1. Move branch master from 62fc20d create test1.txt
+                                to 62fc20d create test1.txt
         2. Move branch master from 62fc20d create test1.txt
                                 to 62fc20d create test1.txt
-        3. Move branch master from 62fc20d create test1.txt
-                                to 62fc20d create test1.txt
+        3. Check out from 62fc20d create test1.txt
+                       to <commit not available: 96d1c37a3d4363611c49f7e52186e189a04c531f>
         Confirm? [yN] branchless: running command: <git-executable> checkout 96d1c37a3d4363611c49f7e52186e189a04c531f --detach
         Failed to check out commit: 96d1c37a3d4363611c49f7e52186e189a04c531f
-        Applied 3 inverse events.
         "###);
         assert!(exit_code > 0);
     }
@@ -842,16 +834,17 @@ fn test_undo_noninteractive() -> eyre::Result<()> {
                 ..Default::default()
             },
         )?;
+        let stdout = trim_lines(stdout);
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
-        1. Check out from 9ed8f9a bad message
-                       to 96d1c37 create test2.txt
-        2. Rewrite commit 9ed8f9a bad message
+        1. Rewrite commit 9ed8f9a bad message
                       as 96d1c37 create test2.txt
-        3. Hide commit 9ed8f9a bad message
-           
-        4. Move branch master from 9ed8f9a bad message
+        2. Hide commit 9ed8f9a bad message
+
+        3. Move branch master from 9ed8f9a bad message
                                 to 96d1c37 create test2.txt
+        4. Check out from 9ed8f9a bad message
+                       to 96d1c37 create test2.txt
         Confirm? [yN] Aborted.
         "###);
     }
@@ -875,22 +868,17 @@ fn test_undo_noninteractive() -> eyre::Result<()> {
         let stdout = trim_lines(stdout);
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
-        1. Check out from 9ed8f9a bad message
-                       to 96d1c37 create test2.txt
-        2. Rewrite commit 9ed8f9a bad message
+        1. Rewrite commit 9ed8f9a bad message
                       as 96d1c37 create test2.txt
-        3. Hide commit 9ed8f9a bad message
+        2. Hide commit 9ed8f9a bad message
 
-        4. Move branch master from 9ed8f9a bad message
+        3. Move branch master from 9ed8f9a bad message
                                 to 96d1c37 create test2.txt
+        4. Check out from 9ed8f9a bad message
+                       to 96d1c37 create test2.txt
         Confirm? [yN] branchless: running command: <git-executable> checkout 96d1c37a3d4363611c49f7e52186e189a04c531f --detach
-        branchless: running command: <git-executable> branchless smartlog
         :
-        O 62fc20d create test1.txt
-        |\
-        | % 96d1c37 (rewritten as 9ed8f9a2) create test2.txt
-        |
-        O 9ed8f9a (master) bad message
+        @ 96d1c37 (master) create test2.txt
         Applied 4 inverse events.
         "###);
     }
@@ -905,3 +893,28 @@ fn test_undo_noninteractive() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_undo_no_confirm() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    if !git.supports_reference_transactions()? {
+        return Ok(());
+    }
+
+    git.init_repo()?;
+    git.commit_file("test1", 1)?;
+
+    {
+        let (stdout, _stderr) = git.run(&["undo", "--yes"])?;
+        let stdout = trim_lines(stdout);
+        insta::assert_snapshot!(stdout, @r###"
+        Will apply these actions:
+        1. Hide commit 62fc20d create test1.txt
+
+        Applied 1 inverse event.
+        "###);
+    }
+
+    Ok(())
+}
